@@ -65,6 +65,7 @@ import {
     AST_Call,
     AST_Case,
     AST_Catch,
+    AST_Chain,
     AST_Class,
     AST_ClassExpression,
     AST_ClassProperty,
@@ -892,11 +893,19 @@ function OutputStream(options) {
 
     PARENS(AST_Arrow, function(output) {
         var p = output.parent();
+
+        if (
+            output.option("wrap_func_args")
+            && p instanceof AST_Call
+            && p.args.includes(this)
+        ) {
+            return true;
+        }
         return p instanceof AST_PropAccess && p.expression === this;
     });
 
-    // same goes for an object literal, because otherwise it would be
-    // interpreted as a block of code.
+    // same goes for an object literal (as in AST_Function), because
+    // otherwise {...} would be interpreted as a block of code.
     PARENS(AST_Object, function(output) {
         return !output.has_parens() && first_in_statement(output);
     });
@@ -962,6 +971,10 @@ function OutputStream(options) {
                 return true;
             }
 
+            if (po === "??" && (so === "||" || so === "&&")) {
+                return true;
+            }
+
             const pp = PRECEDENCE[po];
             const sp = PRECEDENCE[so];
             if (pp > sp
@@ -1412,6 +1425,8 @@ function OutputStream(options) {
             || e instanceof AST_PropAccess
             || e instanceof AST_Unary
             || e instanceof AST_Constant
+            || e instanceof AST_Await
+            || e instanceof AST_Object
         );
         if (parens) output.print("(");
         self.expression.print(output);
@@ -1718,6 +1733,7 @@ function OutputStream(options) {
         if (self.expression instanceof AST_Call || self.expression instanceof AST_Lambda) {
             output.add_mapping(self.start);
         }
+        if (self.optional) output.print("?.");
         output.with_parens(function() {
             self.args.forEach(function(expr, i) {
                 if (i) output.comma();
@@ -1761,6 +1777,9 @@ function OutputStream(options) {
         var print_computed = RESERVED_WORDS.has(prop)
             ? output.option("ie8")
             : !is_identifier_string(prop, output.option("ecma") >= 2015);
+
+        if (self.optional) output.print("?.");
+
         if (print_computed) {
             output.print("[");
             output.add_mapping(self.end);
@@ -1772,7 +1791,7 @@ function OutputStream(options) {
                     output.print(".");
                 }
             }
-            output.print(".");
+            if (!self.optional) output.print(".");
             // the name after dot would be mapped about here.
             output.add_mapping(self.end);
             output.print_name(prop);
@@ -1780,10 +1799,14 @@ function OutputStream(options) {
     });
     DEFPRINT(AST_Sub, function(self, output) {
         self.expression.print(output);
+        if (self.optional) output.print("?.");
         output.print("[");
         self.property.print(output);
         output.print("]");
     });
+    DEFPRINT(AST_Chain, function(self, output) {
+        self.expression.print(output);
+    });
     DEFPRINT(AST_UnaryPrefix, function(self, output) {
         var op = self.operator;
         output.print(op);
