@@ -50,6 +50,9 @@ class JsonWireProtocol extends Transport {
   getElementId(resultValue) {
     return resultValue.ELEMENT;
   }
+  toElement(resultValue) {
+    return {ELEMENT: resultValue};
+  }
 
   isResultSuccess(result) {
     return result && result.status === Errors.StatusCode.SUCCESS;
